/* eslint-disable capitalized-comments */
/* eslint-env mocha */
'use strict';

const assert = require('assert');
const sinon = require('sinon'); // eslint-disable-line node/no-unpublished-require
const common = require('../../lib/common');


describe('common', function () {

  describe('fileScope', function () {
    it('names a file path', function () {
      const filename = 'lib/foo/bar.js';
      const result = common.fileScope(filename)('baz');
      assert.strictEqual(result, 'bar:baz');
    });
    it('names an index path', function () {
      const filename = 'lib/foo/index.js';
      const result = common.fileScope(filename)('baz');
      assert.strictEqual(result, 'foo:baz');
    });
  }); // fileScope

  describe('generateETag', function () {
    it('generates a tag from data', function () {
      const expected = '"RHUvNyculE/SyROjU0LqzN0arxibrlBnazAashP8UGE"';
      const data = 'example data';
      const result = common.generateETag(undefined, undefined, data);
      assert.strictEqual(result, expected);
    });
    it('generates a tag from data and stats', function () {
      const expected = '"mtI0qCyqXsZVfX0Pgi+G6mQM10y9yVyi1NZejXSYttk"';
      const stat = {
        mtimeMs: 1600113038270.2375,
      };
      const data = 'example data';
      const result = common.generateETag(undefined, stat, data);
      assert.strictEqual(result, expected);
    });
  }); // generateETag

  describe('get', function () {
    const def = 'default';
    it('covers missing obj', function () {
      const result = common.get(undefined, undefined, def);
      assert.strictEqual(result, def);
    });
    it('covers missing prop', function () {
      const result = common.get({}, undefined, def);
      assert.strictEqual(result, def);
    });
    it('covers default', function () {
      const result = common.get({ k: 'v' }, 'x', def);
      assert.strictEqual(result, def);
    });
    it('covers prop', function () {
      const result = common.get({ k: 'v' }, 'k', def);
      assert.strictEqual(result, 'v');
    });
  }); // get

  describe('isClientCached', function () {
    let req, modifiedTimeMs, eTag;
    const blueMoon = 'Sat, 31 Oct 2020 07:51:00 GMT';
    const blueMoonMs = 1604130660000;
    const oneDayMs = 86400000;
    const anotherETag = '"RHUvNyculE/SyROjU0LqzN0arxibrlBnazAashP8UGE"';
    beforeEach(function () {
      req = {
        getHeader: sinon.stub(),
      };
      modifiedTimeMs = 0;
      eTag = '"mtI0qCyqXsZVfX0Pgi+G6mQM10y9yVyi1NZejXSYttk"';
    });
    it('no headers, not cached', function () {
      const result = common.isClientCached(req, modifiedTimeMs, eTag);
      assert.strictEqual(result, false);
    });
    it('modified header, not cached', function () {
      req.getHeader.onCall(0).returns(blueMoon);
      modifiedTimeMs = blueMoonMs + oneDayMs;
      const result = common.isClientCached(req, modifiedTimeMs, eTag);
      assert.strictEqual(result, false);
    });
    it('modified header, cached', function () {
      req.getHeader.onCall(0).returns(blueMoon);
      modifiedTimeMs = blueMoonMs - oneDayMs;
      const result = common.isClientCached(req, modifiedTimeMs, eTag);
      assert.strictEqual(result, true);
    });
    it('match header, not matched', function () {
      req.getHeader.onCall(1).returns(anotherETag);
      const result = common.isClientCached(req, modifiedTimeMs, eTag);
      assert.strictEqual(result, false);
    });
    it('match header, matched', function () {
      req.getHeader.onCall(1).returns(`${anotherETag}, ${eTag}, ${anotherETag}`);
      const result = common.isClientCached(req, modifiedTimeMs, eTag);
      assert.strictEqual(result, true);
    });
    it('match header any, matched', function () {
      req.getHeader.onCall(1).returns('*');
      const result = common.isClientCached(req, modifiedTimeMs, eTag);
      assert.strictEqual(result, true);
    });
    it('modified header cached, match header not matched, not cached', function () {
      req.getHeader.onCall(0).returns(blueMoon);
      modifiedTimeMs = blueMoonMs - oneDayMs;
      req.getHeader.onCall(1).returns(`${anotherETag}, ${anotherETag}`);
      const result = common.isClientCached(req, modifiedTimeMs, eTag);
      assert.strictEqual(result, false);
    });
  }); // iscClientCached

  describe('pick', function () {
    it('picks', function () {
      const srcObj = {
        a: 1,
        b: 2,
        c: 3,
      };
      const result = common.pick(srcObj, ['a', 'c', 'd']);
      assert.deepStrictEqual(result, {
        'a': 1,
        'c': 3,
      });
    });
  }); // pick

  describe('requestLogData', function () {
    it('gives data', function () {
      const req = {
        method: 'GET',
        somethingElse: 'blah',
      };
      const result = common.requestLogData(req);
      assert.deepStrictEqual(result, {
        method: 'GET',
      });
    });
  }); // requestLogData

  describe('responseLogData', function () {
    it('gives data', function () {
      const res = {
        getHeaders: () => ({}),
        statusCode: 200,
        blah: 'blah',
      };
      const result = common.responseLogData(res);
      assert.deepStrictEqual(result, {
        headers: {},
        statusCode: 200,
      });
    });
  }); // responseLogData

  describe('setOptions', function () {
    it('sets options', function () {
      const expected = {
        keyOne: 'default',
        keyTwo: 'option',
        keyThree: 'default',
      };
      const target = {};
      const defaultOptions = {
        keyOne: 'default',
        keyTwo: 'default',
        keyThree: 'default',
      };
      const options = {
        keyTwo: 'option',
        keyFour: 'option',
      };
      common.setOptions(target, defaultOptions, options);
      assert.deepStrictEqual(target, expected);
    });
  }); // setOptions

  describe('splitFirst', function () {
    it('splits', function () {
      const expected = ['foo', 'awoo'];
      const src = 'foo?awoo';
      const delim = '?';
      const fill = 'fill';
      const result = common.splitFirst(src, delim, fill);
      assert.deepStrictEqual(result, expected);
    });
    it('fills', function () {
      const expected = ['foo', 'fill'];
      const src = 'foo';
      const delim = '?';
      const fill = 'fill';
      const result = common.splitFirst(src, delim, fill);
      assert.deepStrictEqual(result, expected);
    });
  }); // splitFirst

  describe('mergeEnum', function () {
    it('merges enums', function () {
      const origEnum = {
        ContentType: {
          TextHTML: 'text/html',
        },
        ErrorResponse: {
          BadRequest: {
            statusCode: 400,
            errorMessage: 'Bad Request',
          },
        },
      };
      const newEnum = {
        ContentType: {
          TextPlain: 'text/plain',
        },
        ErrorResponse: {
          BadResponse: {
            statusCode: 401,
            errorMessage: 'Unauthorized',
          },
        },
        Header: {
          Accept: 'accept',
        },
        Value: 'value',
      };
      const mergedEnum = {
        ContentType: {
          TextHTML: 'text/html',
          TextPlain: 'text/plain',
        },
        ErrorResponse: {
          BadRequest: {
            statusCode: 400,
            errorMessage: 'Bad Request',
          },
          BadResponse: {
            statusCode: 401,
            errorMessage: 'Unauthorized',
          },
        },
        Header: {
          Accept: 'accept',
        },
        Value: 'value',
      };
      const result = common.mergeEnum(origEnum, newEnum);
      assert.deepStrictEqual(result, mergedEnum);
    });
  }); // mergeEnum

  describe('requestId', function () {
    it('returns a string', function () {
      const id = common.requestId();
      assert.strictEqual(typeof id, 'string');
      assert(id.length > 0);
    });
  }); // requestId

  describe('ensureLoggerLevels', function () {
    it('adds missing levels', function () {
      const result = common.ensureLoggerLevels();
      assert.deepStrictEqual(result, common.nullLogger);
    });
  }); // ensureLoggerLevels

  describe('httpStatusCodeClass', function () {
    it('works', function () {
      for (const [statusCode, statusClassExpected] of Object.entries({
        200: 2,
        304: 3,
        404: 4,
        500: 5,
      })) {
        const statusClass = common.httpStatusCodeClass(statusCode);
        assert.strictEqual(statusClass, statusClassExpected);
      }
    });
  }); // ensureLoggerLevels

  describe('mergeDeep', function () {
    it('simple merge', function () {
      const o1 = {
        foo1: 'one',
      };
      const o2 = {
        foo2: 'two',
      };
      const o3 = {
        foo3: 'three',
      };
      const expected = {
        foo1: 'one',
        foo2: 'two',
        foo3: 'three',
      };
      const result = common.mergeDeep(o1, o2, o3);
      assert.deepStrictEqual(result, expected);
    });
    it('deep merge', function () {
      const o1 = {
        foo: {
          quux: 1,
        },
        bar: 'baz',
      };
      const o2 = {
        foo: {
          myList: ['fancy'],
          gleep: {
            fraz: 2,
          },
        },
        blah: 'frop',
      };
      const o3 = {
        foo: {
          myList: ['pants'],
          gleep: {
            troz: 'poit',
          },
          narf: 3,
        },
        bar: 'yup',
      };
      const expected = {
        foo: {
          myList: ['fancy', 'pants'],
          gleep: {
            fraz: 2,
            troz: 'poit',
          },
          narf: 3,
          quux: 1,
        },
        bar: 'yup',
        blah: 'frop',
      };
      const result = common.mergeDeep(o1, o2, o3);
      assert.deepStrictEqual(result, expected);
    });
    it('merged object is distinct', function () {
      const o1 = {
        foo: {
          bar: 3,
        },
      };
      const o2 = {
        baz: {
          quux: 4,
        },
      };
      const expected = {
        foo: {
          bar: 3,
        },
        baz: {
          quux: 4,
        },
      };
      const result = common.mergeDeep(o1, o2);
      assert.deepStrictEqual(result, expected);
      const expected2 = {
        foo: {
          bar: 5,
        },
        baz: {
          quux: 5,
        },
      };
      result.foo.bar = 5;
      result.baz.quux = 5;
      assert.deepStrictEqual(result, expected2);
      assert.strictEqual(o1.foo.bar, 3);
      assert.strictEqual(o2.baz.quux, 4);
    });
  }); // mergeDeep

  describe('unfoldHeaderLines', function () {
    it('folds', function () {
      const lines = [
        'Normal-Header: some header data',
        'Folded-Header: more data',
        '   second line of data',
        '   third line of data',
      ];
      const expected = [
        'Normal-Header: some header data',
        'Folded-Header: more data second line of data third line of data',
      ];
      const result = common.unfoldHeaderLines(lines);
      assert.deepStrictEqual(result, expected);
    });
    it('covers no input', function () {
      const lines = undefined;
      const result = common.unfoldHeaderLines();
      assert.deepStrictEqual(result, lines);
    });
  }); // unfoldHeaderLines

});
