/* eslint-disable capitalized-comments */
/* eslint-env mocha */
'use strict';

const assert = require('assert');
const ContentNegotiation = require('../../lib/content-negotiation');
const Enum = require('../../lib/enum');

const TYPES = {
  Any: '*/*',
  ImageAny: 'image/*',
  ImagePng: 'image/png',
  TextAny: 'text/*',
  TextHtml: 'text/html',
  TextPlain: 'text/plain',
  AppJson: 'application/json',
};

describe('ContentNegotiation', function () {
  describe('_unpackAcceptClause', function () {
    it('handles bare type', function () {
      const clause = TYPES.TextPlain;
      const expected = {
        type: TYPES.TextPlain,
        weight: 1.0,
        params: {},
      };
      const result = ContentNegotiation._unpackAcceptClause(clause);
      assert.deepStrictEqual(result, expected);
    });
    it('handles weighted type', function () {
      const clause = ` ${TYPES.TextPlain}; q=0.2`;
      const expected = {
        type: TYPES.TextPlain,
        weight: 0.2,
        params: {},
      };
      const result = ContentNegotiation._unpackAcceptClause(clause);
      assert.deepStrictEqual(result, expected);
    });
    it('handles type with extension parameters', function () {
      const clause = `${TYPES.TextPlain}; format=flowed;charset=UTF-8`;
      const expected = {
        type: TYPES.TextPlain,
        weight: 1.0,
        params: {
          format: 'flowed',
          charset: 'UTF-8',
        },
      };
      const result = ContentNegotiation._unpackAcceptClause(clause);
      assert.deepStrictEqual(result, expected);
    });
    it('handles type with params and weight', function () {
      const clause = `${TYPES.AppJson};charset=UTF-8;q=0.123;ext=local`;
      const expected = {
        type: TYPES.AppJson,
        weight: 0.123,
        params: {
          charset: 'UTF-8',
          ext: 'local',
        },
      };
      const result = ContentNegotiation._unpackAcceptClause(clause);
      assert.deepStrictEqual(result, expected);
    });
    it('ignores empty type', function () {
      const clause = '';
      const expected = undefined;
      const result = ContentNegotiation._unpackAcceptClause(clause);
      assert.deepStrictEqual(result, expected);
    });
    it('ignores weird param', function () {
      const clause = `${TYPES.AppJson};blah`;
      const expected = {
        type: TYPES.AppJson,
        weight: 1.0,
        params: {},
      };
      const result = ContentNegotiation._unpackAcceptClause(clause);
      assert.deepStrictEqual(result, expected);
    });
  }); // _unpackAcceptClause

  describe('_acceptClauses', function () {
    it('orders multiple types properly', function () {
      const acceptHeader = 'text/*;q=0.3, text/html;q=0.7, text/html;level=1,text/html;level=2;q=0.4, */*;q=0.5';
      const expected = [
        { type: TYPES.TextHtml, weight: 1.0, params: { 'level': '1' } },
        { type: TYPES.TextHtml, weight: 0.7, params: {} },
        { type: TYPES.Any, weight: 0.4998, params: {} }, // N.B. Implementation detail
        { type: TYPES.TextHtml, weight: 0.4, params: { 'level': '2' } },
        { type: TYPES.TextAny, weight: 0.2999, params: {} },  // N.B. Implementation detail
      ];
      const result = ContentNegotiation._acceptClauses(acceptHeader);
      assert.deepStrictEqual(result, expected);
    });
    it('does the expected', function () {
      const acceptHeader = 'text/*;q=0.5, text/plain;q=0.7, */*;q=0.2';
      const expected = [
        { type: TYPES.TextPlain, weight: 0.7, params: {} },
        { type: TYPES.TextAny, weight: 0.4999, params: {} }, // N.B. Implementation detail
        { type: TYPES.Any, weight: 0.1998, params: {} }, // N.B. Implementation detail
      ];
      const result = ContentNegotiation._acceptClauses(acceptHeader);
      assert.deepStrictEqual(result, expected);
    });
    it('covers missing data', function () {
      const result = ContentNegotiation._acceptClauses(undefined);
      assert.deepStrictEqual(result, []);
    });
  }); // _acceptClauses

  describe('_matchType', function () {
    it('matches simmple', function () {
      const result = ContentNegotiation._matchType(TYPES.TextPlain, TYPES.TextPlain);
      assert.strictEqual(result, true);
    });
    it('does not match simmple', function () {
      const result = ContentNegotiation._matchType(TYPES.TextPlain, TYPES.TextHtml);
      assert.strictEqual(result, false);
    });
    it('matches subtype wildcard', function () {
      const result = ContentNegotiation._matchType(TYPES.ImageAny, TYPES.ImagePng);
      assert.strictEqual(result, true);
    });
    it('does not match subtype wildcard', function () {
      const result = ContentNegotiation._matchType(TYPES.ImageAny, TYPES.TextPlain);
      assert.strictEqual(result, false);
    });
    it('matches any wildcard', function () {
      const result = ContentNegotiation._matchType(TYPES.Any, TYPES.AppJson);
      assert.strictEqual(result, true);
    });
    it('matches main type', function () {
      const result = ContentNegotiation._matchType('image', TYPES.ImagePng);
      assert.strictEqual(result, true);
    });
    it('does not match main type', function () {
      const result = ContentNegotiation._matchType('image', TYPES.AppJson);
      assert.strictEqual(result, false);
    });
  }); // _matchType

  describe('accept', function () {
    it('accepts anything', function () {
      const types = [ TYPES.AppJson, TYPES.TextPlain ];
      const acceptHeader = undefined;
      const result = ContentNegotiation.accept(types, acceptHeader);
      assert.strictEqual(result, TYPES.AppJson);
    });

    it('picks the most acceptible type', function () {
      const types = [ TYPES.AppJson, TYPES.TextPlain ];
      const acceptHeader = 'text/*;q=0.5, text/html;q=0.6, text/plain;q=0.7, */*;q=0.2, image/png;q=0.0, image/gif;q=0.0';
      const result = ContentNegotiation.accept(types, acceptHeader);
      assert.strictEqual(result, TYPES.TextPlain);
    });
    it('picks the most acceptible type', function () {
      const types = [ TYPES.AppJson, TYPES.TextPlain ];
      const acceptHeader = 'image/png;q=0.0, image/gif;q=0.0, text/html;q=0.6, text/*;q=0.5, text/plain;q=0.7, */*;q=0.7';
      const result = ContentNegotiation.accept(types, acceptHeader);
      assert.strictEqual(result, TYPES.TextPlain);
    });
    it('refuses to pick a zero-weight type', function () {
      const types = [ TYPES.AppJson, TYPES.TextPlain ];
      const acceptHeader = 'application/*;q=0.0, unhandled/type;q=0.7, none/none;q=0.2';
      const result = ContentNegotiation.accept(types, acceptHeader);
      assert.strictEqual(result, undefined);
    });
    it('picks least-wild match', function () {
      const types = [ TYPES.TextHtml, TYPES.TextPlain ];
      const acceptHeader = '*/*;q=0.5, text/plain;q=0.5, text/*;q=0.5';
      const result = ContentNegotiation.accept(types, acceptHeader);
      assert.strictEqual(result, TYPES.TextPlain);
    });
  });  // accept

  describe('preferred', function () {
    it('falls back to identity', function () {
      const encodings = [];
      const header = 'br, gzip';
      const result = ContentNegotiation.preferred(encodings, header);
      assert.deepStrictEqual(result, [Enum.EncodingType.Identity]);
    });
    it('picks appropriate', function () {
      const encodings = ['br', 'identity'];
      const header = 'lz4, br, gzip;q=.5';
      const result = ContentNegotiation.preferred(encodings, header);
      assert.deepStrictEqual(result, [Enum.EncodingType.Brotli, Enum.EncodingType.Identity]);
    });
    it('respects forbidding', function () {
      const encodings = ['br'];
      const header = 'gzip, *;q=0';
      const result = ContentNegotiation.preferred(encodings, header);
      assert.deepStrictEqual(result, []);
    });
    it('respects identity', function () {
      const encodings = ['br'];
      const header = 'gzip, identity;q=0.1, *;q=0';
      const result = ContentNegotiation.preferred(encodings, header);
      assert.deepStrictEqual(result, [Enum.EncodingType.Identity]);
    });
  }); // preferred

}); // ContentNegotiation

